import { asyncDelay, onHomeworkFinish } from "../script/util";
import { MyCocosSceneComponent } from "../script/MyCocosSceneComponent";
import Game, { GAME_STATE, Option } from "./tool/Game";
import pg from "./tool/pg";

const { ccclass, property } = cc._decorator;

@ccclass
export default class SceneComponent extends MyCocosSceneComponent {

    addPreloadImage() {
        // TODO 根据自己的配置预加载图片资源
        this._imageResList.push({ url: this.data.pic_url });
        this._imageResList.push({ url: this.data.pic_url_2 });
    }

    addPreloadAudio() {
        // TODO 根据自己的配置预加载音频资源
        this._audioResList.push({ url: this.data.audio_url });
    }

    addPreloadAnima() {

    }

    onLoadEnd() {
        // TODO 加载完成后的逻辑写在这里, 下面的代码仅供参考
        this.initData();
        this.initView();
        this.initEvent();
        this.log("statr!!!!!!!!!!!!!")
        this.gameStart()
    }
    protected onDestroy(): void {
        pg.event.clear();
    }

    _cantouch = null;
    initData() {
        Game.getIns().init(this.data);
        Game.getIns().reset();
        // 所有全局变量 默认都是null 
        this._cantouch = true;
    }
    private audioId: any;
    async initView() {
        let boat_left = pg.view.find(this, 'layout_bottom/boat_left')
        let boat_right = pg.view.find(this, 'layout_bottom/boat_right')
        pg.view.setString(pg.view.find(boat_left, 'text'), Game.getIns().boatLeft.text)
        pg.view.setString(pg.view.find(boat_right, 'text'), Game.getIns().boatRight.text)
    }
    initEvent() {
        pg.event.on("game_time_over", () => {
            // alert("game_time_over")
            //这里的事件会发送的很早。但是我们需要等待动画执行完了之后再开始后续的内容
            Game.getIns().addPage();
            if (!Game.getIns().isOver) {
                return;
            }
            this.showGameOver();
        })
    }

    gameStart() {
        pg.audio.playLocalAudio(cc.find(`Canvas/res/audio/btn`));
        if (this.audioId > -1) {
            cc.audioEngine.stopAllEffects();
        }
        Game.getIns().state = GAME_STATE.RUNNING;
        this.initFishs();
    }
    initFishs() {
        // 出现鱼
        let fishList = Game.getIns().getFishList();
        let layout_fish = pg.view.find(this, 'layout_bottom/layout_fish')
        let item = pg.view.find(this, 'layout_bottom/item')
        fishList.forEach((fish, index) => {
            let node = cc.instantiate(item);
            layout_fish.addChild(node);
            this.updateItem(node, fish);
        });
    }
    updateItem(item, data: Option) {
        item.data = data;
        console.log(data);
        let text = pg.view.find(item, 'layout_text/text')
        pg.view.setString(text, data.txt)
        let count = data.id % 10 + 1;
        for (let i = 1; i <= 10; i++) {
            let fishNode = pg.view.find(item, `fish/fish_${i}`)
            fishNode.active = i == count
        }
        //左往右
        if (Math.random() < 0.5) {
            item.x = -(1920 / 2 + Math.random() * 200)
            item.y = -315 - Math.random() * 150;
            item.scaleX = -1;
            text.scaleX = -1;
            cc.tween(item).repeatForever(cc.tween().by(7 + Math.random() * 8, { x: 1920 + 400 }).call(() => {
                item.scaleX = -item.scaleX;
                text.scaleX = -text.scaleX;
            }).by(7 + Math.random() * 8, { x: - 1920 - 400 }).call(() => {
                item.scaleX = -item.scaleX;
                text.scaleX = -text.scaleX;
            })).start();
        } else {
            item.x = 1920 / 2 + Math.random() * 200
            item.y = -315 - Math.random() * 150;
            item.scaleX = 1;
            text.scaleX = 1;
            cc.tween(item).repeatForever(cc.tween().by(7 + Math.random() * 8, { x: -1920 - 400 }).call(() => {
                item.scaleX = -item.scaleX;
                text.scaleX = -text.scaleX;
            }).by(7 + Math.random() * 8, { x: 1920 + 400 }).call(() => {
                item.scaleX = -item.scaleX;
                text.scaleX = -text.scaleX;
            })).start();
        }
        item.on(cc.Node.EventType.TOUCH_START, this.onItemTouchStart, this);
        item.on(cc.Node.EventType.TOUCH_MOVE, this.onItemTouchMove, this);
        item.on(cc.Node.EventType.TOUCH_END, this.onItemTouchEnd, this);
        item.on(cc.Node.EventType.TOUCH_CANCEL, this.onItemTouchCancel, this);
    }
    onTouchItem(e) {
        let item = e.target;
        let data = item.data;
        item.x = 0;
        item.y = 0;
        cc.Tween.stopAllByTarget(item);
        setTimeout(() => {
            this.updateItem(item, data);
        }, 100);
    }
    private setTouchPos(e) {
        let item = e.target;
        let location = e.getLocation();
        cc.Tween.stopAllByTarget(item);
        let pos = location.sub(cc.v2(1920 / 2, 1080 / 2));
        item.x = pos.x;
        item.y = pos.y;
    }
    onItemTouchStart(e) {
        let target: cc.Node = e.target;
        let data = target.data;
        if (data.isChecked) return;
        this.setTouchPos(e);
    }
    onItemTouchMove(e) {
        let target: cc.Node = e.target;
        let data = target.data;
        if (data.isChecked) return;
        this.setTouchPos(e);
    }
    onItemTouchEnd(e) {
        //
        let target: cc.Node = e.target;
        let data = target.data;
        if (data.isChecked) return;
        // let rect = cc.rect(target.x, target.y, target.width, target.height);
        let rect = cc.rect(target.x, target.y, 50, 50);

        let boat_left = pg.view.find(this, 'layout_bottom/boat_left')
        let boat_right = pg.view.find(this, 'layout_bottom/boat_right')
        let leftRect = cc.rect(boat_left.x - boat_left.width / 2, boat_left.y - boat_left.height / 2, boat_left.width, boat_left.height);
        let rightRect = cc.rect(boat_right.x - boat_right.width / 2, boat_right.y - boat_right.height / 2, boat_right.width, boat_right.height);

        let isRight = false;
        let currentRect = null;
        if (rect.intersects(leftRect)) {
            isRight = !!Game.getIns().boatLeft.isChild(data);
            Game.getIns().boatLeft.setRight(data);
            currentRect = leftRect;
        } else if (rect.intersects(rightRect)) {
            isRight = !!Game.getIns().boatRight.isChild(data);
            Game.getIns().boatRight.setRight(data);
            currentRect = rightRect;
        }
        if (isRight) {
            // target
            let item = target;
            let lastArea = 100;
            if (item.scaleX == -1) {
                // 计算当前坐标距离最左侧的距离
                let space = currentRect.width - ((rect.x + rect.width) - currentRect.x) - lastArea / 5;
                let time = space / 10;
                cc.tween(item).by(time, { x: space }).start();
            } else {
                // 计算当前坐标距离最左侧的距离
                let space = rect.x - currentRect.x - lastArea;
                let time = space / 10;
                cc.tween(item).by(time, { x: -space }).start();
            }
        } else {
            this.updateItem(target, data);
        }
        //判断是否结束了
        if (Game.getIns().boatLeft.isAllRight() && Game.getIns().boatRight.isAllRight()) {
            this.showGameOver();
        }
    }
    onItemTouchCancel() {

    }


    private showGameOver() {
        let gameover = cc.find("gameover", this.node);
        gameover.scale = 3;
        gameover.active = true;
        this.playLocalAudio("gameover");
        cc.tween(gameover).to(0.3, { scale: 1 })
            .delay(2)
            .call(() => {
                gameover.active = false;
                //发送给上端最后的数据
                // alert("game_finish")
                let player = { right: Game.getIns().player.score, error: Game.getIns().player.error }
                onHomeworkFinish(player)
                this.log("player: " + JSON.stringify(player));
            }).start();
    }

    playLocalAudio(audioName) {
        const audio = cc.find(`Canvas/res/audio/${audioName}`).getComponent(cc.AudioSource);
        return new Promise((resolve, reject) => {
            const id = cc.audioEngine.playEffect(audio.clip, false);
            cc.audioEngine.setFinishCallback(id, () => { resolve(id); });
        })
    }
}


